# Copyright 2018, Cossack Labs Limited
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# coding: utf-8

from sqlalchemy import types
from acrawriter import create_acrastruct


__all__ = ('AcraBinary', 'AcraString')


class AcraBinary(types.TypeDecorator):
    impl = types.Binary

    def __init__(self, public_key, *args, **kwargs):
        super(AcraBinary, self).__init__(*args, **kwargs)
        self._public_key = public_key

    def process_bind_param(self, value, dialect):
        return create_acrastruct(value, self._public_key)

    def process_result_value(self, value, dialect):
        return value


class AcraString(AcraBinary):
    def __init__(self, public_key, encoding='utf-8', *args, **kwargs):
        super(AcraString, self).__init__(public_key, *args, **kwargs)
        self._encoding = encoding

    def process_bind_param(self, value, dialect):
        return super(AcraString, self).process_bind_param(
            value.encode(self._encoding), dialect)

    def process_result_value(self, value, dialect):
        if isinstance(value, str):
            return value
        else:
            return value.decode(self._encoding)



# Copyright 2016, Cossack Labs Limited
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# coding: utf-8
import sys
import struct
from random import randint

from pythemis.scell import SCellSeal
from pythemis.skeygen import GenerateKeyPair, KEY_PAIR_TYPE
from pythemis.smessage import SMessage

__all__ = ('create_acrastruct')

BEGIN_TAG = [ord('"')]*8

if sys.version[0] == 3:
    BEGIN_TAG = bytes(BEGIN_TAG)
    def generate_key():
        return bytes([randint(0, 255) for _ in range(SYMMETRIC_KEY_LENGTH)])
else:
    BEGIN_TAG = bytes(bytearray(BEGIN_TAG))
    def generate_key():
        return bytes(bytearray([randint(0, 255) for _ in range(SYMMETRIC_KEY_LENGTH)]))

SYMMETRIC_KEY_LENGTH = 32


def create_acrastruct(data, acra_public_key, context=None):
    random_kp = GenerateKeyPair(KEY_PAIR_TYPE.EC)
    smessage = SMessage(random_kp.export_private_key(), acra_public_key)
    random_key = generate_key()
    wrapped_random_key = smessage.wrap(random_key)

    scell = SCellSeal(random_key)
    encrypted_data = scell.encrypt(data, context)
    del random_key
    encrypted_data_len = struct.pack('<Q', len(encrypted_data))

    acrastruct = (
        BEGIN_TAG +
        random_kp.export_public_key() +
        wrapped_random_key +
        encrypted_data_len + encrypted_data
    )
    del random_kp
    del wrapped_random_key
    return acrastruct
create_acrastruct.__annotations__ = {
    'data': bytes, 'acra_public_key': bytes, 'context': bytes
}


# Copyright 2016, Cossack Labs Limited
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# coding: utf-8
from django.core import validators
from django.db import models
from django import forms
from django.utils import six
from django.conf import settings
from django.utils.translation import ugettext_lazy as _
import acrawriter

__all__ = ('CharField', 'EmailField', 'TextField')


class CharField(models.CharField):
    def __init__(self, public_key=None, encoding='utf-8',
                 encoding_errors='ignore', *args, **kwargs):
        super(CharField, self).__init__(*args, **kwargs)
        self._encoding = encoding
        self._encoding_errors = encoding_errors
        if not (public_key or settings.ACRA_SERVER_PUBLIC_KEY):
            raise ValueError("Set public key arg or settings.ACRA_SERVER_PUBLIC_KEY")
        self._public_key = public_key or settings.ACRA_SERVER_PUBLIC_KEY

    def from_db_value(self, value, *args, **kwargs):
        if isinstance(value, memoryview):
            value = value.tobytes()
        if isinstance(value, six.binary_type):
            return value.decode(self._encoding, errors=self._encoding_errors)
        else:
            return value

    def get_db_prep_value(self, value, connection, prepared=False):
        value = super(CharField, self).get_db_prep_value(
            value, connection, prepared)
        if value == '':
            return b''
        elif value is None:
            return None
        else:
            return acrawriter.create_acrastruct(value.encode(self._encoding), self._public_key)

    def get_internal_type(self):
        return 'BinaryField'

    def to_python(self, value):
        value = super(CharField, self).to_python(value)
        if isinstance(value, six.binary_type):
            return value.decode(self._encoding, errors=self._encoding_errors)
        else:
            return value


class EmailField(CharField):
    default_validators = [validators.validate_email]
    description = _("Email address")

    def __init__(self, *args, **kwargs):
        kwargs['max_length'] = kwargs.get('max_length', 254)
        super(EmailField, self).__init__(*args, **kwargs)


class TextField(CharField):
    description = _("Text")

    def __init__(self, *args, **kwargs):
        super(TextField, self).__init__(*args, **kwargs)
        self.validators = []

    def formfield(self, **kwargs):
        # Passing max_length to forms.CharField means that the value's length
        # will be validated twice. This is considered acceptable since we want
        # the value in the form field (to pass into widget for example).
        defaults = {'max_length': self.max_length, 'widget': forms.Textarea}
        defaults.update(kwargs)
        return super(TextField, self).formfield(**defaults)

    def check(self, **kwargs):
        return []



# Copyright 2016, Cossack Labs Limited
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#!/usr/bin/env python
from setuptools import setup

setup(name='acrawriter',
      version='1.0.2',
      description='AcraWriter library for Python: encrypts data into AcraStructs, allowing Acra to decrypt it',
      author='Cossack Labs',
      author_email='dev@cossacklabs.com',
      url='https://github.com/cossacklabs/acra/',
      packages=['acrawriter', 'acrawriter.django', 'acrawriter.sqlalchemy'],
      install_requires=[
          'pythemis',
      ],
      classifiers=[
          'Programming Language :: Python :: 2',
          'Programming Language :: Python :: 2.6',
          'Programming Language :: Python :: 2.7',
          'Programming Language :: Python :: 3',
          'Programming Language :: Python :: 3.2',
          'Programming Language :: Python :: 3.3',
          'Programming Language :: Python :: 3.4',
          'Programming Language :: Python :: 3.5',
      ],)
